/*     */ package org.hibernate.cfg;
/*     */ 
/*     */ import java.util.ArrayList;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Properties;
/*     */ import java.util.Set;
/*     */ import javax.persistence.Embeddable;
/*     */ import javax.persistence.Entity;
/*     */ import javax.persistence.MappedSuperclass;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.annotations.AnyMetaDef;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionManager;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.engine.NamedQueryDefinition;
/*     */ import org.hibernate.engine.NamedSQLQueryDefinition;
/*     */ import org.hibernate.engine.ResultSetMappingDefinition;
/*     */ import org.hibernate.mapping.IdGenerator;
/*     */ import org.hibernate.mapping.Join;
/*     */ import org.hibernate.mapping.PersistentClass;
/*     */ import org.hibernate.mapping.Table;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ExtendedMappings
/*     */   extends Mappings
/*     */ {
/*  37 */   private static final Log log = LogFactory.getLog(ExtendedMappings.class);
/*     */   
/*     */ 
/*     */   private final Map<String, IdGenerator> namedGenerators;
/*     */   
/*     */   private final Map<String, Map<String, Join>> joins;
/*     */   
/*     */   private final Map<String, AnnotatedClassType> classTypes;
/*     */   
/*     */   private final Map<String, Properties> generatorTables;
/*     */   
/*     */   private final Map<Table, List<String[]>> tableUniqueConstraints;
/*     */   
/*     */   private final Map<String, String> mappedByResolver;
/*     */   
/*     */   private final Map<String, String> propertyRefResolver;
/*     */   
/*     */   private final ReflectionManager reflectionManager;
/*     */   
/*     */   private final Set<String> defaultNamedQueryNames;
/*     */   
/*     */   private final Set<String> defaultNamedNativeQueryNames;
/*     */   
/*     */   private final Set<String> defaultSqlResulSetMappingNames;
/*     */   
/*     */   private final Set<String> defaultNamedGenerators;
/*     */   
/*     */   private final Map<String, AnyMetaDef> anyMetaDefs;
/*     */   
/*     */ 
/*     */   ExtendedMappings(Map classes, Map collections, Map tables, Map queries, Map sqlqueries, Map sqlResultSetMappings, Set<String> defaultNamedQueryNames, Set<String> defaultNamedNativeQueryNames, Set<String> defaultSqlResulSetMappingNames, Set<String> defaultNamedGenerators, Map imports, List secondPasses, List propertyReferences, NamingStrategy namingStrategy, Map typeDefs, Map filterDefinitions, Map namedGenerators, Map<String, Map<String, Join>> joins, Map<String, AnnotatedClassType> classTypes, Map extendsQueue, Map<String, Mappings.TableDescription> tableNameBinding, Map<Table, Mappings.ColumnNames> columnNameBindingPerTable, List auxiliaryDatabaseObjects, Map<String, Properties> generatorTables, Map<Table, List<String[]>> tableUniqueConstraints, Map<String, String> mappedByResolver, Map<String, String> propertyRefResolver, Map<String, AnyMetaDef> anyMetaDefs, ReflectionManager reflectionManager)
/*     */   {
/*  69 */     super(classes, collections, tables, queries, sqlqueries, sqlResultSetMappings, imports, secondPasses, propertyReferences, namingStrategy, typeDefs, filterDefinitions, extendsQueue, auxiliaryDatabaseObjects, tableNameBinding, columnNameBindingPerTable);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  87 */     this.namedGenerators = namedGenerators;
/*  88 */     this.joins = joins;
/*  89 */     this.classTypes = classTypes;
/*  90 */     this.generatorTables = generatorTables;
/*  91 */     this.tableUniqueConstraints = tableUniqueConstraints;
/*  92 */     this.mappedByResolver = mappedByResolver;
/*  93 */     this.propertyRefResolver = propertyRefResolver;
/*  94 */     this.reflectionManager = reflectionManager;
/*  95 */     this.defaultNamedQueryNames = defaultNamedQueryNames;
/*  96 */     this.defaultNamedNativeQueryNames = defaultNamedNativeQueryNames;
/*  97 */     this.defaultSqlResulSetMappingNames = defaultSqlResulSetMappingNames;
/*  98 */     this.defaultNamedGenerators = defaultNamedGenerators;
/*  99 */     this.anyMetaDefs = anyMetaDefs;
/*     */   }
/*     */   
/*     */   public void addGenerator(IdGenerator generator) throws MappingException {
/* 103 */     if (!this.defaultNamedGenerators.contains(generator.getName())) {
/* 104 */       Object old = this.namedGenerators.put(generator.getName(), generator);
/* 105 */       if (old != null) log.warn("duplicate generator name: " + generator.getName());
/*     */     }
/*     */   }
/*     */   
/*     */   public void addJoins(PersistentClass persistentClass, Map<String, Join> joins) throws MappingException {
/* 110 */     Object old = this.joins.put(persistentClass.getEntityName(), joins);
/* 111 */     if (old != null) log.warn("duplicate joins for class: " + persistentClass.getEntityName());
/*     */   }
/*     */   
/*     */   public AnnotatedClassType addClassType(XClass clazz) { AnnotatedClassType type;
/*     */     AnnotatedClassType type;
/* 116 */     if (clazz.isAnnotationPresent(Entity.class)) {
/* 117 */       type = AnnotatedClassType.ENTITY;
/*     */     } else { AnnotatedClassType type;
/* 119 */       if (clazz.isAnnotationPresent(Embeddable.class)) {
/* 120 */         type = AnnotatedClassType.EMBEDDABLE;
/*     */       } else { AnnotatedClassType type;
/* 122 */         if (clazz.isAnnotationPresent(MappedSuperclass.class)) {
/* 123 */           type = AnnotatedClassType.EMBEDDABLE_SUPERCLASS;
/*     */         }
/*     */         else
/* 126 */           type = AnnotatedClassType.NONE;
/*     */       } }
/* 128 */     this.classTypes.put(clazz.getName(), type);
/* 129 */     return type;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public AnnotatedClassType getClassType(XClass clazz)
/*     */   {
/* 137 */     AnnotatedClassType type = (AnnotatedClassType)this.classTypes.get(clazz.getName());
/* 138 */     if (type == null) {
/* 139 */       return addClassType(clazz);
/*     */     }
/*     */     
/* 142 */     return type;
/*     */   }
/*     */   
/*     */   public IdGenerator getGenerator(String name)
/*     */   {
/* 147 */     return getGenerator(name, null);
/*     */   }
/*     */   
/*     */   public Map<String, Join> getJoins(String persistentClass) {
/* 151 */     return (Map)this.joins.get(persistentClass);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public IdGenerator getGenerator(String name, Map<String, IdGenerator> localGenerators)
/*     */   {
/* 163 */     if (localGenerators != null) {
/* 164 */       IdGenerator result = (IdGenerator)localGenerators.get(name);
/* 165 */       if (result != null) return result;
/*     */     }
/* 167 */     return (IdGenerator)this.namedGenerators.get(name);
/*     */   }
/*     */   
/*     */   public void addGeneratorTable(String name, Properties params) {
/* 171 */     Object old = this.generatorTables.put(name, params);
/* 172 */     if (old != null) log.warn("duplicate generator table: " + name);
/*     */   }
/*     */   
/*     */   public Properties getGeneratorTableProperties(String name, Map<String, Properties> localGeneratorTables) {
/* 176 */     if (localGeneratorTables != null) {
/* 177 */       Properties result = (Properties)localGeneratorTables.get(name);
/* 178 */       if (result != null) return result;
/*     */     }
/* 180 */     return (Properties)this.generatorTables.get(name);
/*     */   }
/*     */   
/*     */   public void addUniqueConstraints(Table table, List uniqueConstraints) {
/* 184 */     List oldConstraints = (List)this.tableUniqueConstraints.get(table);
/* 185 */     if (oldConstraints == null) {
/* 186 */       oldConstraints = new ArrayList();
/* 187 */       this.tableUniqueConstraints.put(table, oldConstraints);
/*     */     }
/* 189 */     oldConstraints.addAll(uniqueConstraints);
/*     */   }
/*     */   
/*     */   public Map<Table, List<String[]>> getTableUniqueConstraints() {
/* 193 */     return this.tableUniqueConstraints;
/*     */   }
/*     */   
/*     */   public void addMappedBy(String entityName, String propertyName, String inversePropertyName) {
/* 197 */     this.mappedByResolver.put(entityName + "." + propertyName, inversePropertyName);
/*     */   }
/*     */   
/*     */   public String getFromMappedBy(String entityName, String propertyName) {
/* 201 */     return (String)this.mappedByResolver.get(entityName + "." + propertyName);
/*     */   }
/*     */   
/*     */   public void addPropertyReferencedAssociation(String entityName, String propertyName, String propertyRef) {
/* 205 */     this.propertyRefResolver.put(entityName + "." + propertyName, propertyRef);
/*     */   }
/*     */   
/*     */   public String getPropertyReferencedAssociation(String entityName, String propertyName) {
/* 209 */     return (String)this.propertyRefResolver.get(entityName + "." + propertyName);
/*     */   }
/*     */   
/*     */   public void addUniquePropertyReference(String referencedClass, String propertyName)
/*     */   {
/* 214 */     super.addUniquePropertyReference(referencedClass, propertyName);
/*     */   }
/*     */   
/*     */   public void addPropertyReference(String referencedClass, String propertyName)
/*     */   {
/* 219 */     super.addPropertyReference(referencedClass, propertyName);
/*     */   }
/*     */   
/*     */   public ReflectionManager getReflectionManager() {
/* 223 */     return this.reflectionManager;
/*     */   }
/*     */   
/*     */   public void addDefaultQuery(String name, NamedQueryDefinition query) {
/* 227 */     super.addQuery(name, query);
/* 228 */     this.defaultNamedQueryNames.add(name);
/*     */   }
/*     */   
/*     */   public void addDefaultSQLQuery(String name, NamedSQLQueryDefinition query) {
/* 232 */     super.addSQLQuery(name, query);
/* 233 */     this.defaultNamedNativeQueryNames.add(name);
/*     */   }
/*     */   
/*     */   public void addDefaultGenerator(IdGenerator idGen) {
/* 237 */     addGenerator(idGen);
/* 238 */     this.defaultNamedGenerators.add(idGen.getName());
/*     */   }
/*     */   
/*     */   public void addDefaultResultSetMapping(ResultSetMappingDefinition definition)
/*     */   {
/* 243 */     String name = definition.getName();
/* 244 */     if ((!this.defaultSqlResulSetMappingNames.contains(name)) && (super.getResultSetMapping(name) != null))
/*     */     {
/* 246 */       this.resultSetMappings.remove(name);
/*     */     }
/* 248 */     super.addResultSetMapping(definition);
/* 249 */     this.defaultSqlResulSetMappingNames.add(name);
/*     */   }
/*     */   
/*     */   public void addQuery(String name, NamedQueryDefinition query) throws MappingException
/*     */   {
/* 254 */     if (!this.defaultNamedQueryNames.contains(name)) super.addQuery(name, query);
/*     */   }
/*     */   
/*     */   public void addResultSetMapping(ResultSetMappingDefinition definition)
/*     */   {
/* 259 */     if (!this.defaultSqlResulSetMappingNames.contains(definition.getName())) {
/* 260 */       super.addResultSetMapping(definition);
/*     */     }
/*     */   }
/*     */   
/*     */   public void addSQLQuery(String name, NamedSQLQueryDefinition query) throws MappingException {
/* 265 */     if (!this.defaultNamedNativeQueryNames.contains(name)) super.addSQLQuery(name, query);
/*     */   }
/*     */   
/*     */   public Map getClasses() {
/* 269 */     return this.classes;
/*     */   }
/*     */   
/*     */   public void addAnyMetaDef(AnyMetaDef defAnn) {
/* 273 */     if (this.anyMetaDefs.containsKey(defAnn.name())) {
/* 274 */       throw new AnnotationException("Two @AnyMetaDef with the same name defined: " + defAnn.name());
/*     */     }
/* 276 */     this.anyMetaDefs.put(defAnn.name(), defAnn);
/*     */   }
/*     */   
/*     */   public AnyMetaDef getAnyMetaDef(String name) {
/* 280 */     return (AnyMetaDef)this.anyMetaDefs.get(name);
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\ExtendedMappings.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */