/*     */ package org.hibernate.cfg.search;
/*     */ 
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.Properties;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AnnotationException;
/*     */ import org.hibernate.event.EventListeners;
/*     */ import org.hibernate.event.PostDeleteEventListener;
/*     */ import org.hibernate.event.PostInsertEventListener;
/*     */ import org.hibernate.event.PostUpdateEventListener;
/*     */ import org.hibernate.util.ReflectHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SearchConfiguration
/*     */ {
/*  22 */   private static Log log = LogFactory.getLog(SearchConfiguration.class);
/*     */   
/*     */   public static void enableHibernateSearch(EventListeners eventListeners, Properties properties)
/*     */   {
/*  26 */     boolean enableSearchListeners = !"false".equalsIgnoreCase(properties.getProperty("hibernate.search.autoregister_listeners"));
/*  27 */     boolean enableCollectionSearchListeners = false;
/*  28 */     Class searchEventListenerClass = null;
/*  29 */     Class nonCollectionSearchEventListener = null;
/*     */     try {
/*  31 */       searchEventListenerClass = ReflectHelper.classForName("org.hibernate.search.event.FullTextIndexEventListener", SearchConfiguration.class);
/*     */       
/*     */ 
/*  34 */       nonCollectionSearchEventListener = searchEventListenerClass;
/*     */       try
/*     */       {
/*  37 */         ReflectHelper.classForName("org.hibernate.event.AbstractCollectionEvent", SearchConfiguration.class);
/*     */         
/*     */ 
/*     */ 
/*  41 */         searchEventListenerClass = ReflectHelper.classForName("org.hibernate.search.event.FullTextIndexCollectionEventListener", SearchConfiguration.class);
/*     */         
/*     */ 
/*     */ 
/*  45 */         enableCollectionSearchListeners = true;
/*     */ 
/*     */       }
/*     */       catch (ClassNotFoundException e)
/*     */       {
/*  50 */         log.debug("Hibernate Search collection listeners not present (upgrate to Hibernate Core 3.2.6 and above and Hibernate Search 3.0.1 and above)");
/*     */       }
/*     */       
/*     */     }
/*     */     catch (ClassNotFoundException e)
/*     */     {
/*  56 */       log.debug("Search not present in classpath, ignoring event listener registration");
/*     */     }
/*  58 */     if ((enableSearchListeners) && (searchEventListenerClass != null))
/*     */     {
/*     */       Object searchEventListener;
/*     */       try {
/*  62 */         searchEventListener = searchEventListenerClass.newInstance();
/*     */       }
/*     */       catch (Exception e) {
/*  65 */         throw new AnnotationException("Unable to load Search event listener", e);
/*     */       }
/*     */       
/*  68 */       boolean present = false;
/*  69 */       PostInsertEventListener[] listeners = eventListeners.getPostInsertEventListeners();
/*  70 */       if (listeners != null) {
/*  71 */         for (Object eventListener : listeners)
/*     */         {
/*  73 */           present = (present) || (searchEventListenerClass == eventListener.getClass()) || ((enableCollectionSearchListeners) && (nonCollectionSearchEventListener == eventListener.getClass()));
/*     */         }
/*     */         
/*     */ 
/*  77 */         if (!present) {
/*  78 */           int length = listeners.length + 1;
/*  79 */           PostInsertEventListener[] newListeners = new PostInsertEventListener[length];
/*  80 */           System.arraycopy(listeners, 0, newListeners, 0, length - 1);
/*  81 */           newListeners[(length - 1)] = ((PostInsertEventListener)searchEventListener);
/*  82 */           eventListeners.setPostInsertEventListeners(newListeners);
/*     */         }
/*     */       }
/*     */       else {
/*  86 */         eventListeners.setPostInsertEventListeners(new PostInsertEventListener[] { (PostInsertEventListener)searchEventListener });
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*  92 */       boolean present = false;
/*  93 */       PostUpdateEventListener[] listeners = eventListeners.getPostUpdateEventListeners();
/*  94 */       if (listeners != null) {
/*  95 */         for (Object eventListener : listeners)
/*     */         {
/*  97 */           present = (present) || (searchEventListenerClass == eventListener.getClass()) || ((enableCollectionSearchListeners) && (nonCollectionSearchEventListener == eventListener.getClass()));
/*     */         }
/*     */         
/*     */ 
/* 101 */         if (!present) {
/* 102 */           int length = listeners.length + 1;
/* 103 */           PostUpdateEventListener[] newListeners = new PostUpdateEventListener[length];
/* 104 */           System.arraycopy(listeners, 0, newListeners, 0, length - 1);
/* 105 */           newListeners[(length - 1)] = ((PostUpdateEventListener)searchEventListener);
/* 106 */           eventListeners.setPostUpdateEventListeners(newListeners);
/*     */         }
/*     */       }
/*     */       else {
/* 110 */         eventListeners.setPostUpdateEventListeners(new PostUpdateEventListener[] { (PostUpdateEventListener)searchEventListener });
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 116 */       boolean present = false;
/* 117 */       PostDeleteEventListener[] listeners = eventListeners.getPostDeleteEventListeners();
/* 118 */       if (listeners != null) {
/* 119 */         for (Object eventListener : listeners)
/*     */         {
/* 121 */           present = (present) || (searchEventListenerClass == eventListener.getClass()) || ((enableCollectionSearchListeners) && (nonCollectionSearchEventListener == eventListener.getClass()));
/*     */         }
/*     */         
/*     */ 
/* 125 */         if (!present) {
/* 126 */           int length = listeners.length + 1;
/* 127 */           PostDeleteEventListener[] newListeners = new PostDeleteEventListener[length];
/* 128 */           System.arraycopy(listeners, 0, newListeners, 0, length - 1);
/* 129 */           newListeners[(length - 1)] = ((PostDeleteEventListener)searchEventListener);
/* 130 */           eventListeners.setPostDeleteEventListeners(newListeners);
/*     */         }
/*     */       }
/*     */       else {
/* 134 */         eventListeners.setPostDeleteEventListeners(new PostDeleteEventListener[] { (PostDeleteEventListener)searchEventListener });
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 139 */       if (enableCollectionSearchListeners) {
/*     */         try {
/* 141 */           Class collectionSearchConfigurationClass = ReflectHelper.classForName("org.hibernate.cfg.search.CollectionSearchConfiguration", SearchConfiguration.class);
/*     */           
/*     */ 
/* 144 */           Method method = collectionSearchConfigurationClass.getDeclaredMethod("enableHibernateSearch", new Class[] { EventListeners.class, Object.class, Class.class });
/*     */           
/* 146 */           method.invoke(null, new Object[] { eventListeners, searchEventListener, searchEventListenerClass });
/*     */         }
/*     */         catch (Exception e) {
/* 149 */           throw new AnnotationException("Unable to load Search event listener", e);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-annotations-3.3.1.GA.jar!\org\hibernate\cfg\search\SearchConfiguration.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */