/*     */ package org.hibernate.ejb;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.ObjectInputStream;
/*     */ import java.io.ObjectOutputStream;
/*     */ import java.io.Serializable;
/*     */ import java.util.Map;
/*     */ import javax.persistence.EntityExistsException;
/*     */ import javax.persistence.EntityNotFoundException;
/*     */ import javax.persistence.EntityTransaction;
/*     */ import javax.persistence.FlushModeType;
/*     */ import javax.persistence.LockModeType;
/*     */ import javax.persistence.NoResultException;
/*     */ import javax.persistence.OptimisticLockException;
/*     */ import javax.persistence.PersistenceContextType;
/*     */ import javax.persistence.PersistenceException;
/*     */ import javax.persistence.TransactionRequiredException;
/*     */ import javax.persistence.spi.PersistenceUnitTransactionType;
/*     */ import javax.transaction.Synchronization;
/*     */ import javax.transaction.SystemException;
/*     */ import javax.transaction.TransactionManager;
/*     */ import org.apache.commons.logging.Log;
/*     */ import org.apache.commons.logging.LogFactory;
/*     */ import org.hibernate.AssertionFailure;
/*     */ import org.hibernate.FlushMode;
/*     */ import org.hibernate.HibernateException;
/*     */ import org.hibernate.LockMode;
/*     */ import org.hibernate.MappingException;
/*     */ import org.hibernate.ObjectDeletedException;
/*     */ import org.hibernate.ObjectNotFoundException;
/*     */ import org.hibernate.QueryException;
/*     */ import org.hibernate.SQLQuery;
/*     */ import org.hibernate.Session;
/*     */ import org.hibernate.SessionFactory;
/*     */ import org.hibernate.StaleObjectStateException;
/*     */ import org.hibernate.StaleStateException;
/*     */ import org.hibernate.TransientObjectException;
/*     */ import org.hibernate.TypeMismatchException;
/*     */ import org.hibernate.UnresolvableObjectException;
/*     */ import org.hibernate.ejb.transaction.JoinableCMTTransaction;
/*     */ import org.hibernate.ejb.transaction.JoinableCMTTransaction.JoinStatus;
/*     */ import org.hibernate.ejb.util.ConfigurationHelper;
/*     */ import org.hibernate.engine.SessionFactoryImplementor;
/*     */ import org.hibernate.engine.SessionImplementor;
/*     */ import org.hibernate.exception.ConstraintViolationException;
/*     */ import org.hibernate.proxy.HibernateProxy;
/*     */ import org.hibernate.transaction.TransactionFactory.Context;
/*     */ import org.hibernate.util.CollectionHelper;
/*     */ import org.hibernate.util.JTAHelper;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractEntityManagerImpl
/*     */   implements HibernateEntityManagerImplementor, Serializable
/*     */ {
/*  65 */   private static Log log = LogFactory.getLog(AbstractEntityManagerImpl.class);
/*     */   
/*  67 */   protected transient TransactionImpl tx = new TransactionImpl(this);
/*     */   protected PersistenceContextType persistenceContextType;
/*  69 */   private FlushModeType flushModeType = FlushModeType.AUTO;
/*     */   private PersistenceUnitTransactionType transactionType;
/*     */   private Map properties;
/*     */   
/*     */   protected AbstractEntityManagerImpl(PersistenceContextType type, PersistenceUnitTransactionType transactionType, Map properties)
/*     */   {
/*  75 */     this.persistenceContextType = type;
/*  76 */     this.transactionType = transactionType;
/*  77 */     this.properties = (properties != null ? properties : CollectionHelper.EMPTY_MAP);
/*     */   }
/*     */   
/*     */   protected void postInit()
/*     */   {
/*  82 */     if (PersistenceUnitTransactionType.JTA.equals(this.transactionType)) joinTransaction(true);
/*  83 */     Object flushMode = this.properties.get("org.hibernate.flushMode");
/*  84 */     if (flushMode != null) {
/*  85 */       getSession().setFlushMode(ConfigurationHelper.getFlushMode(flushMode));
/*     */     }
/*  87 */     this.properties = null;
/*     */   }
/*     */   
/*     */   public javax.persistence.Query createQuery(String ejbqlString)
/*     */   {
/*     */     try {
/*  93 */       return new QueryImpl(getSession().createQuery(ejbqlString), this);
/*     */     }
/*     */     catch (HibernateException he) {
/*  96 */       throwPersistenceException(he); }
/*  97 */     return null;
/*     */   }
/*     */   
/*     */   public javax.persistence.Query createNamedQuery(String name)
/*     */   {
/*     */     org.hibernate.Query namedQuery;
/*     */     try
/*     */     {
/* 105 */       namedQuery = getSession().getNamedQuery(name);
/*     */     }
/*     */     catch (MappingException e) {
/* 108 */       throw new IllegalArgumentException("Named query not found: " + name);
/*     */     }
/*     */     try {
/* 111 */       return new QueryImpl(namedQuery, this);
/*     */     }
/*     */     catch (HibernateException he) {
/* 114 */       throwPersistenceException(he); }
/* 115 */     return null;
/*     */   }
/*     */   
/*     */   public javax.persistence.Query createNativeQuery(String sqlString)
/*     */   {
/*     */     try
/*     */     {
/* 122 */       SQLQuery q = getSession().createSQLQuery(sqlString);
/* 123 */       return new QueryImpl(q, this);
/*     */     }
/*     */     catch (HibernateException he) {
/* 126 */       throwPersistenceException(he); }
/* 127 */     return null;
/*     */   }
/*     */   
/*     */   public javax.persistence.Query createNativeQuery(String sqlString, Class resultClass)
/*     */   {
/*     */     try
/*     */     {
/* 134 */       SQLQuery q = getSession().createSQLQuery(sqlString);
/* 135 */       q.addEntity("alias1", resultClass.getName(), LockMode.READ);
/* 136 */       return new QueryImpl(q, this);
/*     */     }
/*     */     catch (HibernateException he) {
/* 139 */       throwPersistenceException(he); }
/* 140 */     return null;
/*     */   }
/*     */   
/*     */   public javax.persistence.Query createNativeQuery(String sqlString, String resultSetMapping)
/*     */   {
/*     */     try
/*     */     {
/* 147 */       SQLQuery q = getSession().createSQLQuery(sqlString);
/* 148 */       q.setResultSetMapping(resultSetMapping);
/* 149 */       return new QueryImpl(q, this);
/*     */     }
/*     */     catch (HibernateException he) {
/* 152 */       throwPersistenceException(he); }
/* 153 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   public <T> T getReference(Class<T> entityClass, Object primaryKey)
/*     */   {
/*     */     try
/*     */     {
/* 161 */       return (T)getSession().load(entityClass, (Serializable)primaryKey);
/*     */     }
/*     */     catch (MappingException e) {
/* 164 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (TypeMismatchException e) {
/* 167 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (ClassCastException e) {
/* 170 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 173 */       throwPersistenceException(he); }
/* 174 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   public <A> A find(Class<A> entityClass, Object primaryKey)
/*     */   {
/*     */     try
/*     */     {
/* 182 */       return (A)getSession().get(entityClass, (Serializable)primaryKey);
/*     */     }
/*     */     catch (ObjectDeletedException e)
/*     */     {
/* 186 */       return null;
/*     */     }
/*     */     catch (ObjectNotFoundException e)
/*     */     {
/* 190 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (MappingException e) {
/* 193 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (TypeMismatchException e) {
/* 196 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (ClassCastException e) {
/* 199 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 202 */       throwPersistenceException(he); }
/* 203 */     return null;
/*     */   }
/*     */   
/*     */   private void checkTransactionNeeded()
/*     */   {
/* 208 */     if ((this.persistenceContextType == PersistenceContextType.TRANSACTION) && (!isTransactionInProgress()))
/*     */     {
/* 210 */       throw new TransactionRequiredException("no transaction is in progress for a TRANSACTION type persistence context");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */   public void persist(Object entity)
/*     */   {
/* 217 */     checkTransactionNeeded();
/*     */     try
/*     */     {
/* 220 */       getSession().persist(entity);
/*     */     }
/*     */     catch (MappingException e) {
/* 223 */       throw new IllegalArgumentException(e.getMessage());
/*     */     }
/*     */     catch (HibernateException he) {
/* 226 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */   public <A> A merge(A entity)
/*     */   {
/* 232 */     checkTransactionNeeded();
/*     */     try
/*     */     {
/* 235 */       return (A)getSession().merge(entity);
/*     */     }
/*     */     catch (ObjectDeletedException sse) {
/* 238 */       throw new IllegalArgumentException(sse);
/*     */     }
/*     */     catch (MappingException e) {
/* 241 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 244 */       throwPersistenceException(he); }
/* 245 */     return null;
/*     */   }
/*     */   
/*     */   public void remove(Object entity)
/*     */   {
/* 250 */     checkTransactionNeeded();
/*     */     try
/*     */     {
/* 253 */       getSession().delete(entity);
/*     */     }
/*     */     catch (MappingException e) {
/* 256 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 259 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */   public void refresh(Object entity) {
/* 264 */     checkTransactionNeeded();
/*     */     try
/*     */     {
/* 267 */       if (!getSession().contains(entity)) {
/* 268 */         throw new IllegalArgumentException("Entity not managed");
/*     */       }
/* 270 */       getSession().refresh(entity);
/*     */     }
/*     */     catch (MappingException e) {
/* 273 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 276 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean contains(Object entity) {
/*     */     try {
/* 282 */       if ((entity != null) && (!(entity instanceof HibernateProxy)) && (getSession().getSessionFactory().getClassMetadata(entity.getClass()) == null))
/*     */       {
/*     */ 
/* 285 */         throw new IllegalArgumentException("Not an entity:" + entity.getClass());
/*     */       }
/* 287 */       return getSession().contains(entity);
/*     */     }
/*     */     catch (MappingException e) {
/* 290 */       throw new IllegalArgumentException(e.getMessage(), e);
/*     */     }
/*     */     catch (HibernateException he) {
/* 293 */       throwPersistenceException(he); }
/* 294 */     return false;
/*     */   }
/*     */   
/*     */   public void flush()
/*     */   {
/*     */     try {
/* 300 */       if (!isTransactionInProgress()) {
/* 301 */         throw new TransactionRequiredException("no transaction is in progress");
/*     */       }
/*     */       
/* 304 */       getSession().flush();
/*     */     }
/*     */     catch (HibernateException he) {
/* 307 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public abstract Session getSession();
/*     */   
/*     */ 
/*     */ 
/*     */   protected abstract Session getRawSession();
/*     */   
/*     */ 
/*     */ 
/*     */   public EntityTransaction getTransaction()
/*     */   {
/* 323 */     if (this.transactionType == PersistenceUnitTransactionType.JTA) {
/* 324 */       throw new IllegalStateException("A JTA EntityManager cannot use getTransaction()");
/*     */     }
/* 326 */     return this.tx;
/*     */   }
/*     */   
/*     */   public void setFlushMode(FlushModeType flushModeType) {
/* 330 */     this.flushModeType = flushModeType;
/* 331 */     if (flushModeType == FlushModeType.AUTO) {
/* 332 */       getSession().setFlushMode(FlushMode.AUTO);
/*     */     }
/* 334 */     else if (flushModeType == FlushModeType.COMMIT) {
/* 335 */       getSession().setFlushMode(FlushMode.COMMIT);
/*     */     }
/*     */     else {
/* 338 */       throw new AssertionFailure("Unknown FlushModeType: " + flushModeType);
/*     */     }
/*     */   }
/*     */   
/*     */   public void clear()
/*     */   {
/*     */     try {
/* 345 */       getSession().clear();
/*     */     }
/*     */     catch (HibernateException he) {
/* 348 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */   public FlushModeType getFlushMode() {
/* 353 */     FlushMode mode = getSession().getFlushMode();
/* 354 */     if (mode == FlushMode.AUTO) {
/* 355 */       this.flushModeType = FlushModeType.AUTO;
/*     */     }
/* 357 */     else if (mode == FlushMode.COMMIT) {
/* 358 */       this.flushModeType = FlushModeType.COMMIT;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 370 */       return null;
/*     */     }
/*     */     
/* 373 */     return this.flushModeType;
/*     */   }
/*     */   
/*     */   public void lock(Object entity, LockModeType lockMode) {
/*     */     try {
/* 378 */       if (!isTransactionInProgress()) {
/* 379 */         throw new TransactionRequiredException("no transaction is in progress");
/*     */       }
/*     */       
/* 382 */       if (!contains(entity)) throw new IllegalArgumentException("entity not in the persistence context");
/* 383 */       getSession().lock(entity, getLockMode(lockMode));
/*     */     }
/*     */     catch (HibernateException he) {
/* 386 */       throwPersistenceException(he);
/*     */     }
/*     */   }
/*     */   
/*     */   private LockMode getLockMode(LockModeType lockMode) {
/* 391 */     switch (lockMode) {
/*     */     case READ: 
/* 393 */       return LockMode.UPGRADE;
/*     */     case WRITE: 
/* 395 */       return LockMode.FORCE;
/*     */     }
/* 397 */     throw new AssertionFailure("Unknown LockModeType: " + lockMode);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void adjustFlushMode()
/*     */   {
/* 406 */     Session session = getSession();
/*     */     
/* 408 */     boolean isTransactionActive = isTransactionInProgress();
/*     */     
/* 410 */     if ((isTransactionActive) && (session.getFlushMode() == FlushMode.MANUAL)) {
/* 411 */       log.debug("Transaction activated, move to FlushMode " + this.flushModeType);
/* 412 */       setFlushMode(this.flushModeType);
/*     */     }
/* 414 */     else if ((!isTransactionActive) && (session.getFlushMode() != FlushMode.MANUAL)) {
/* 415 */       log.debug("Transaction not active, move to FlushMode NEVER");
/* 416 */       session.setFlushMode(FlushMode.MANUAL);
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isTransactionInProgress() {
/* 421 */     return ((SessionImplementor)getRawSession()).isTransactionInProgress();
/*     */   }
/*     */   
/*     */   protected void markAsRollback() {
/* 425 */     log.debug("mark transaction for rollback");
/* 426 */     if (this.tx.isActive()) {
/* 427 */       this.tx.setRollbackOnly();
/*     */ 
/*     */ 
/*     */     }
/* 431 */     else if (PersistenceUnitTransactionType.JTA == this.transactionType) {
/* 432 */       TransactionManager transactionManager = ((SessionFactoryImplementor)getRawSession().getSessionFactory()).getTransactionManager();
/*     */       
/* 434 */       if (transactionManager == null) {
/* 435 */         throw new PersistenceException("Using a JTA persistence context wo setting hibernate.transaction.manager_lookup_class");
/*     */       }
/*     */       
/*     */       try
/*     */       {
/* 440 */         transactionManager.setRollbackOnly();
/*     */       }
/*     */       catch (SystemException e) {
/* 443 */         throw new PersistenceException("Unable to set the JTA transaction as RollbackOnly", e);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   public void joinTransaction()
/*     */   {
/* 450 */     joinTransaction(false);
/*     */   }
/*     */   
/*     */   private void joinTransaction(boolean ignoreNotJoining)
/*     */   {
/* 455 */     getSession().isOpen();
/* 456 */     if (this.transactionType == PersistenceUnitTransactionType.JTA) {
/*     */       try {
/* 458 */         log.debug("Looking for a JTA transaction to join");
/* 459 */         final Session session = getSession();
/* 460 */         org.hibernate.Transaction transaction = session.getTransaction();
/* 461 */         if ((transaction != null) && ((transaction instanceof JoinableCMTTransaction)))
/*     */         {
/* 463 */           JoinableCMTTransaction joinableCMTTransaction = (JoinableCMTTransaction)transaction;
/*     */           
/* 465 */           if (joinableCMTTransaction.getStatus() == JoinableCMTTransaction.JoinStatus.JOINED) {
/* 466 */             log.debug("Transaction already joined");
/* 467 */             return;
/*     */           }
/* 469 */           joinableCMTTransaction.markForJoined();
/* 470 */           session.isOpen();
/* 471 */           if (joinableCMTTransaction.getStatus() == JoinableCMTTransaction.JoinStatus.NOT_JOINED) {
/* 472 */             if (ignoreNotJoining) {
/* 473 */               log.debug("No JTA transaction found");
/* 474 */               return;
/*     */             }
/*     */             
/* 477 */             throw new TransactionRequiredException("No active JTA transaction on joinTransaction call");
/*     */           }
/*     */           
/*     */ 
/*     */ 
/*     */ 
/* 483 */           if (joinableCMTTransaction.getStatus() == JoinableCMTTransaction.JoinStatus.MARKED_FOR_JOINED) {
/* 484 */             throw new AssertionFailure("Transaction MARKED_FOR_JOINED after isOpen() call");
/*     */           }
/*     */           
/*     */ 
/* 488 */           log.trace("Adding flush() and close() synchronization");
/* 489 */           joinableCMTTransaction.registerSynchronization(new Synchronization()
/*     */           {
/*     */             public void beforeCompletion() {
/* 492 */               boolean flush = false;
/* 493 */               TransactionFactory.Context ctx = null;
/*     */               try {
/* 495 */                 ctx = (TransactionFactory.Context)session;
/* 496 */                 JoinableCMTTransaction joinable = (JoinableCMTTransaction)session.getTransaction();
/* 497 */                 javax.transaction.Transaction transaction = joinable.getTransaction();
/* 498 */                 if (transaction == null)
/* 499 */                   AbstractEntityManagerImpl.log.warn("Transaction not available on beforeCompletionPhase: assuming valid");
/* 500 */                 flush = (!ctx.isFlushModeNever()) && ((transaction == null) || (!JTAHelper.isRollback(transaction.getStatus())));
/*     */ 
/*     */ 
/*     */               }
/*     */               catch (SystemException se)
/*     */               {
/*     */ 
/* 507 */                 AbstractEntityManagerImpl.log.error("could not determine transaction status", se);
/*     */                 
/* 509 */                 AbstractEntityManagerImpl.this.throwPersistenceException(new PersistenceException("could not determine transaction status in beforeCompletion()", se));
/*     */ 
/*     */ 
/*     */               }
/*     */               catch (HibernateException he)
/*     */               {
/*     */ 
/*     */ 
/* 517 */                 AbstractEntityManagerImpl.this.throwPersistenceException(he);
/*     */               }
/*     */               try
/*     */               {
/* 521 */                 if (flush) {
/* 522 */                   AbstractEntityManagerImpl.log.trace("automatically flushing session");
/* 523 */                   ctx.managedFlush();
/*     */                 }
/*     */                 else {
/* 526 */                   AbstractEntityManagerImpl.log.trace("skipping managed flushing");
/*     */                 }
/*     */               }
/*     */               catch (RuntimeException re)
/*     */               {
/* 531 */                 if ((re instanceof HibernateException)) {
/* 532 */                   AbstractEntityManagerImpl.this.throwPersistenceException((HibernateException)re);
/*     */                 }
/*     */                 else {
/* 535 */                   AbstractEntityManagerImpl.this.throwPersistenceException(new PersistenceException(re));
/*     */                 }
/*     */               }
/*     */             }
/*     */             
/*     */             public void afterCompletion(int status) {
/*     */               try {
/* 542 */                 if ((4 == status) && (AbstractEntityManagerImpl.this.transactionType == PersistenceUnitTransactionType.JTA))
/*     */                 {
/* 544 */                   if (session.isOpen()) {
/* 545 */                     session.clear();
/*     */                   }
/*     */                 }
/* 548 */                 if (session.isOpen())
/*     */                 {
/* 550 */                   JoinableCMTTransaction joinable = (JoinableCMTTransaction)session.getTransaction();
/* 551 */                   joinable.resetStatus();
/*     */                 }
/*     */               }
/*     */               catch (HibernateException e) {
/* 555 */                 AbstractEntityManagerImpl.this.throwPersistenceException(e);
/*     */               }
/*     */             }
/*     */           });
/*     */         }
/*     */         else
/*     */         {
/* 562 */           log.warn("Cannot join transaction: do not override hibernate.transaction.factory_class");
/*     */         }
/*     */       }
/*     */       catch (HibernateException he) {
/* 566 */         throwPersistenceException(he);
/*     */       }
/*     */     }
/*     */     
/* 570 */     if (!ignoreNotJoining) { log.warn("Calling joinTransaction() on a non JTA EntityManager");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public Object getDelegate()
/*     */   {
/* 578 */     return getSession();
/*     */   }
/*     */   
/*     */   private void writeObject(ObjectOutputStream oos)
/*     */     throws IOException
/*     */   {
/* 584 */     oos.defaultWriteObject();
/*     */   }
/*     */   
/*     */   private void readObject(ObjectInputStream ois) throws IOException, ClassNotFoundException {
/* 588 */     ois.defaultReadObject();
/* 589 */     this.tx = new TransactionImpl(this);
/*     */   }
/*     */   
/*     */   public void throwPersistenceException(PersistenceException e) {
/* 593 */     if ((!(e instanceof NoResultException)) && (!(e instanceof javax.persistence.NonUniqueResultException))) {
/*     */       try {
/* 595 */         markAsRollback();
/*     */       }
/*     */       catch (Exception ne)
/*     */       {
/* 599 */         log.error("Unable to mark for rollback on PersistenceException: ", ne);
/*     */       }
/*     */     }
/* 602 */     throw e;
/*     */   }
/*     */   
/*     */   public void throwPersistenceException(HibernateException e) {
/* 606 */     if ((e instanceof StaleStateException)) {
/* 607 */       PersistenceException pe = wrapStaleStateException((StaleStateException)e);
/* 608 */       throwPersistenceException(pe);
/*     */     }
/* 610 */     else if ((e instanceof ConstraintViolationException))
/*     */     {
/* 612 */       throwPersistenceException(new EntityExistsException(e));
/*     */     }
/* 614 */     else if ((e instanceof ObjectNotFoundException)) {
/* 615 */       throwPersistenceException(new EntityNotFoundException(e.getMessage()));
/*     */     }
/* 617 */     else if ((e instanceof org.hibernate.NonUniqueResultException)) {
/* 618 */       throwPersistenceException(new javax.persistence.NonUniqueResultException(e.getMessage()));
/*     */     }
/* 620 */     else if ((e instanceof UnresolvableObjectException)) {
/* 621 */       throwPersistenceException(new EntityNotFoundException(e.getMessage()));
/*     */     } else {
/* 623 */       if ((e instanceof QueryException)) {
/* 624 */         throw new IllegalArgumentException(e);
/*     */       }
/* 626 */       if ((e instanceof TransientObjectException)) {
/*     */         try {
/* 628 */           markAsRollback();
/*     */         }
/*     */         catch (Exception ne)
/*     */         {
/* 632 */           log.error("Unable to mark for rollback on TransientObjectException: ", ne);
/*     */         }
/* 634 */         throw new IllegalStateException(e);
/*     */       }
/*     */       
/* 637 */       throwPersistenceException(new PersistenceException(e));
/*     */     }
/*     */   }
/*     */   
/*     */   public PersistenceException wrapStaleStateException(StaleStateException e) { PersistenceException pe;
/*     */     PersistenceException pe;
/* 643 */     if ((e instanceof StaleObjectStateException)) {
/* 644 */       StaleObjectStateException sose = (StaleObjectStateException)e;
/* 645 */       Serializable identifier = sose.getIdentifier();
/* 646 */       PersistenceException pe; if (identifier != null) {
/* 647 */         Object entity = getRawSession().load(sose.getEntityName(), identifier);
/* 648 */         PersistenceException pe; if ((entity instanceof Serializable))
/*     */         {
/* 650 */           pe = new OptimisticLockException(null, e, entity);
/*     */         }
/*     */         else {
/* 653 */           pe = new OptimisticLockException(e);
/*     */         }
/*     */       }
/*     */       else {
/* 657 */         pe = new OptimisticLockException(e);
/*     */       }
/*     */     }
/*     */     else {
/* 661 */       pe = new OptimisticLockException(e);
/*     */     }
/* 663 */     return pe;
/*     */   }
/*     */ }


/* Location:              C:\Users\anandkum\Desktop\source\hibernate-entitymanager-3.3.2.GA.jar!\org\hibernate\ejb\AbstractEntityManagerImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */