package com.netapp.myasup.reports.design;

import java.awt.Color;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExpression;
import net.sf.jasperreports.engine.JRField;
import net.sf.jasperreports.engine.design.JRDesignBand;
import net.sf.jasperreports.engine.design.JRDesignExpression;
import net.sf.jasperreports.engine.design.JRDesignField;
import net.sf.jasperreports.engine.design.JRDesignFrame;
import net.sf.jasperreports.engine.design.JRDesignGroup;
import net.sf.jasperreports.engine.design.JRDesignQuery;
import net.sf.jasperreports.engine.design.JRDesignSection;
import net.sf.jasperreports.engine.design.JRDesignStyle;
import net.sf.jasperreports.engine.design.JRDesignTextField;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.type.EvaluationTimeEnum;
import net.sf.jasperreports.engine.type.HorizontalAlignEnum;
import net.sf.jasperreports.engine.type.HyperlinkTargetEnum;
import net.sf.jasperreports.engine.type.HyperlinkTypeEnum;
import net.sf.jasperreports.engine.type.ModeEnum;
import net.sf.jasperreports.engine.type.PositionTypeEnum;
import net.sf.jasperreports.engine.type.SplitTypeEnum;
import net.sf.jasperreports.engine.type.StretchTypeEnum;
import net.sf.jasperreports.engine.type.WhenNoDataTypeEnum;
import net.sf.jasperreports.engine.xml.JRXmlLoader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.netapp.myasup.jasper.constants.RMApplicationConstants;
import com.netapp.myasup.jasper.reportGenerator.handler.FieldHandler;

public class ReportCreator extends GenericReportCreator {
	private final Log LOG = LogFactory.getLog(getClass());

	public ReportCreator() {

	}

	public JasperDesign getJasperDesign1(String reportName, String fileName,String[] fieldOrder,String outputFormat,String htmlMarkupFields) {
		// The basic structure
				LOG.info("reportName " + reportName);
				LOG.info("fileName " + fileName);
				//LOG.info("fieldOrder " + fieldOrder.length);
				
				JasperDesign jasperDesign = createBasicStructure(reportName, fileName);
				
				LOG.info("jasperDesign " + jasperDesign);
				LOG.info("Width " + jasperDesign.getPageWidth());
				
//				
				//Removing Title band
				if("CSV".equals(outputFormat)){
					jasperDesign.setTitle(null);
//					jasperDesign.getT
				}
				
				FieldHandler fieldHandler = new FieldHandler();
				Map<String, JRField> fieldsMap = fieldHandler.getFieldMap(jasperDesign);
				java.util.Set<String> fields = fieldsMap.keySet();
				//end
				LOG.info("fieldsMap " + fieldsMap.size());
				
				int pageWidth = (fields.size() * RMApplicationConstants.COLUMN_WIDTH_NORMAL) + 100;
				LOG.info("pageWidth " + pageWidth);
				if(pageWidth < 1000){
					pageWidth = 1000;
				}
				
				jasperDesign.setPageWidth(pageWidth);

				JRDesignBand columnHeaderBand = (JRDesignBand) jasperDesign.getColumnHeader();
				JRDesignBand detailBand = (JRDesignBand) (jasperDesign.getDetailSection().getBands()[0]);
				int x = 0;
				int y = 0;
				
        	for (String key : fieldOrder) {
				
					LOG.info("key "+key);
					JRField jrField = fieldsMap.get(key);
					JRDesignTextField designTextField = createTextField(x, y, RMApplicationConstants.COLUMN_WIDTH_NORMAL, RMApplicationConstants.COLUMN_HEIGHT, jrField.getDescription(), jrField
							.getValueClassName(), fieldHandler.getLabelValue(jrField.getName()), HorizontalAlignEnum.LEFT, null);
					
					designTextField.setFontSize(14);
					designTextField.setBold(true);
					
					JRDesignTextField detailTextField = createTextField(x, y, RMApplicationConstants.COLUMN_WIDTH_NORMAL, RMApplicationConstants.COLUMN_HEIGHT, jrField.getDescription(), jrField
							.getValueClassName(), fieldHandler.getExpressionValue(jrField.getName()), HorizontalAlignEnum.LEFT, null);
					
					detailTextField.setFontSize(12);
					LOG.info("htmlMarkupFields" + htmlMarkupFields);
					if(htmlMarkupFields.contains(key)){
						LOG.info("Marup Set for key "+key);
						detailTextField.setMarkup("html");
//						designTextField.setMarkup("styled");
					}
					
					if(jrField.getName().equalsIgnoreCase("INTERNAL INFO")) {
						JRDesignExpression internalUserExpression = new JRDesignExpression();
						internalUserExpression.setText(" new Boolean($P{userdn}.equalsIgnoreCase( \"internal\" ))");;
						designTextField.setPrintWhenExpression(internalUserExpression);
						detailTextField.setPrintWhenExpression(internalUserExpression);
					}
					
					columnHeaderBand.addElement(designTextField);
					detailBand.addElement(detailTextField);



					x += RMApplicationConstants.COLUMN_WIDTH_NORMAL;
				}
				LOG.info("Page Width " + jasperDesign.getPageWidth());
				//jasperDesign.setColumnHeader(columnHeaderBand);
				
							
				LOG.info("End Jasper Design ");
				
				return (jasperDesign);

	}
	public JasperDesign getJasperDesign(String reportName, String fileName,String[] fieldOrder,String outputFormat,String htmlMarkupFields) {
		// The basic structure
		LOG.info("reportName " + reportName);
		LOG.info("fileName " + fileName);
		//LOG.info("fieldOrder " + fieldOrder.length);
		
		JasperDesign jasperDesign = createBasicStructure(reportName, fileName);
		
		LOG.info("jasperDesign " + jasperDesign);
		LOG.info("Width " + jasperDesign.getPageWidth());
		
//		for (JRDesignStyle jrDesignStyle : createStyles()) {
//            try {
//                jasperDesign.addStyle(jrDesignStyle);
//            } catch (JRException e) {
//                LOG.error("Error in adding a style to the report... "+e.getMessage());
//            }
//        }
		
		//Removing Title band
		if("CSV".equals(outputFormat)){
			jasperDesign.setTitle(null);
//			jasperDesign.getT
		}else{
			JRDesignBand titleBand = (JRDesignBand) jasperDesign.getTitle();
		}
		
		FieldHandler fieldHandler = new FieldHandler();
		Map<String, JRField> fieldsMap = fieldHandler.getFieldMap(jasperDesign);

		//Changes by yamar - removing the use of fieldOrder parameter, the required field info is available from jasperDesign fieldMap
		java.util.Set<String> fields = fieldsMap.keySet();
		//end
		LOG.info("fieldsMap " + fieldsMap.size());
		
		int pageWidth = (fields.size() * RMApplicationConstants.COLUMN_WIDTH_NORMAL) + 100;
		LOG.info("pageWidth " + pageWidth);
		if(pageWidth < 1000){
			pageWidth = 1000;
		}
		
		jasperDesign.setPageWidth(pageWidth);

		JRDesignBand columnHeaderBand = (JRDesignBand) jasperDesign.getColumnHeader();

		int x = 0;
		int y = 0;

//		for (String key : fieldsMap.keySet()) {
		//Changes by yamar - removing the use of fieldOrder parameter, the required field info is available from jasperDesign fieldMap		
		for (String key : fields) {
		//end	
//			LOG.info("x" + x);
			LOG.info("key "+key);
			JRField jrField = fieldsMap.get(key);

//			LOG.info("jrField.getName() "+jrField.getName());
//			LOG.info("jrField.getDescription() "+jrField.getDescription());
//			LOG.info("jrField.getValueClassName() "+jrField.getValueClassName());
			// LOG.info(jrField.getDescription());
			// LOG.info(jrField.getValueClassName());

			JRDesignTextField designTextField = createTextField(x, y, RMApplicationConstants.COLUMN_WIDTH_NORMAL, RMApplicationConstants.COLUMN_HEIGHT, jrField.getDescription(), jrField
					.getValueClassName(), fieldHandler.getLabelValue(jrField.getName()), HorizontalAlignEnum.LEFT, null);
			
			designTextField.setFontSize(14);
			designTextField.setBold(true);
			
			columnHeaderBand.addElement(designTextField);

			x += RMApplicationConstants.COLUMN_WIDTH_NORMAL;
		}
		LOG.info("Page Width " + jasperDesign.getPageWidth());
		jasperDesign.setColumnHeader(columnHeaderBand);
		
		JRDesignBand detailBand = (JRDesignBand) (jasperDesign.getDetailSection().getBands()[0]);

		// detail.setSplitType(SplitTypeEnum.STRETCH);
		// detail.setHeight(12);

		x = 0;
		y = 0;

		for (String key : fieldOrder) {

			LOG.info("x" + x);

			JRField jrField = fieldsMap.get(key);

			// LOG.info(jrField.getName());
			// LOG.info(jrField.getDescription());
			// LOG.info(jrField.getValueClassName());
			//        	
//			JRDesignTextField designTextField = createTextField(x, y, RMApplicationConstants.COLUMN_WIDTH_NORMAL, RMApplicationConstants.COLUMN_HEIGHT, jrField.getDescription(), jrField
//					.getValueClassName(), fieldHandler.getExpressionValue(jrField.getName()), HorizontalAlignEnum.LEFT, (JRDesignStyle) jasperDesign.getStylesMap().get("detailStyle"));
			JRDesignTextField designTextField = createTextField(x, y, RMApplicationConstants.COLUMN_WIDTH_NORMAL, RMApplicationConstants.COLUMN_HEIGHT, jrField.getDescription(), jrField
					.getValueClassName(), fieldHandler.getExpressionValue(jrField.getName()), HorizontalAlignEnum.LEFT, null);
			
			designTextField.setFontSize(12);
			LOG.info("htmlMarkupFields" + htmlMarkupFields);
			if(htmlMarkupFields.contains(key)){
				LOG.info("Marup Set for key "+key);
				designTextField.setMarkup("html");
//				designTextField.setMarkup("styled");
			}
			
			detailBand.addElement(designTextField);

			x += RMApplicationConstants.COLUMN_WIDTH_NORMAL;
		}
		
		
		LOG.info("End Jasper Design ");
		
		return (jasperDesign);
	}

	/**
	 * Creates the report basic structure
	 * 
	 * @param reportName
	 *            ReportName
	 * @return The JasperDesign object
	 */
	public JasperDesign createBasicStructure(String reportName, String fileName) {
		// The report design
		JasperDesign jasperDesign = null;
		try {
		    InputStream fileInputStream = this.getClass().getClassLoader().getResourceAsStream(fileName);
			LOG.info("fileInputStream " + fileInputStream);
			jasperDesign = JRXmlLoader.load(fileInputStream);
			
			/*FileManager fileManager = new FileManager();
			
			jasperDesign = (JasperDesign) fileManager.readFile(fileName);*/ 
			
			LOG.info("jasperDesign " + jasperDesign);

			jasperDesign.setName(reportName);
			jasperDesign.setColumnCount(1);
			jasperDesign.setPageWidth(612);
			jasperDesign.setPageHeight(792);
			jasperDesign.setColumnWidth(562);
			jasperDesign.setColumnSpacing(0);
			jasperDesign.setLeftMargin(30);
			jasperDesign.setRightMargin(20);
			jasperDesign.setTopMargin(30);
			jasperDesign.setBottomMargin(30);
			jasperDesign.setWhenNoDataType(WhenNoDataTypeEnum.ALL_SECTIONS_NO_DETAIL);
			jasperDesign.setTitleNewPage(false);
			jasperDesign.setSummaryNewPage(false);

		} catch (JRException e) {
			e.printStackTrace();
		}
		return (jasperDesign);
	}

	/**
	 * Creates a list of Report Styles (only one for this sample)
	 * 
	 * @return a list of JRDesignStyle objects
	 */
	private List<JRDesignStyle> createStyles() {
		List<JRDesignStyle> stylesList = new ArrayList<JRDesignStyle>();
		// title style
		JRDesignStyle titleStyle = createStyle("titleStyle", "DejaVu Sans", Color.BLACK, Color.WHITE, 20, true, false, false);
		// detail style
		JRDesignStyle detailStyle = createStyle("detailStyle", "DejaVu Sans", Color.BLACK, Color.WHITE, 12, false, false, false);

		// group header style
		JRDesignStyle headerStyle = createStyle("headerStyle", "DejaVu Sans", Color.WHITE, Color.GRAY, 12, true, false, false);

		// group footer style
		JRDesignStyle footerStyle = createStyle("footerStyle", "DejaVu Sans", Color.WHITE, Color.BLACK, 12, true, false, false);
		stylesList.add(titleStyle);
		stylesList.add(headerStyle);
		stylesList.add(detailStyle);
		stylesList.add(footerStyle);

		return (stylesList);
	}

	/**
	 * Creates the title Band
	 * 
	 * @param jrStyle
	 *            The titleStyle
	 * @return The title band of the report
	 */
	private JRDesignBand createTitleBand(JRDesignStyle jrStyle) {
		JRDesignBand designBand = new JRDesignBand();
		designBand.setHeight(20);

		// Create a TextField (textFieldTag attributes)
		JRDesignTextField textField = new JRDesignTextField();
		textField.setStretchWithOverflow(true);
		textField.setBlankWhenNull(true);
		textField.setEvaluationTime(EvaluationTimeEnum.NOW);
		textField.setHyperlinkType(HyperlinkTypeEnum.NONE);
		textField.setHyperlinkTarget(HyperlinkTargetEnum.SELF);

		textField.setStyle(jrStyle);
		textField.setX(0);
		textField.setY(0);
		textField.setWidth(500);
		textField.setHeight(14);
		textField.setKey("titleTextField");
		textField.setHorizontalAlignment(HorizontalAlignEnum.CENTER);
		textField.setStretchType(StretchTypeEnum.RELATIVE_TO_TALLEST_OBJECT);
		textField.setPositionType(PositionTypeEnum.FLOAT);
		textField.setMode(ModeEnum.OPAQUE);
		// box tag properties
		textField.getLineBox().setPadding(1);

		// TextField expression
		JRDesignExpression designExpression = new JRDesignExpression();
		designExpression.setValueClass(String.class);
		designExpression.setText("\"Cars report\"");

		// Setting the expression
		textField.setExpression(designExpression);

		// Add the textField to the band
		designBand.addElement(textField);
		return (designBand);
	}

	/**
	 * Creates que sql query
	 * 
	 * @return The sql query
	 */
	private JRDesignQuery createQuery() {
		JRDesignQuery query = new JRDesignQuery();
		query.setLanguage("ejbql");
		query.setText("SELECT car.model, car.color, person.id, person.name, person.age " + "FROM Car car JOIN car.owner person " + "ORDER BY person.id, car.model");
		return (query);
	}

	/**
	 * Create a list with the report fields for the query
	 * 
	 * @return The report fields list
	 */
	private List<JRDesignField> getFieldList() {
		List<JRDesignField> fieldsList = new ArrayList<JRDesignField>();
		fieldsList.add(createReportField("carModel", String.class.getName(), "COLUMN_1"));
		fieldsList.add(createReportField("carColor", String.class.getName(), "COLUMN_2"));
		fieldsList.add(createReportField("personId", Integer.class.getName(), "COLUMN_3"));
		fieldsList.add(createReportField("personName", String.class.getName(), "COLUMN_4"));
		fieldsList.add(createReportField("personAge", Integer.class.getName(), "COLUMN_5"));
		return (fieldsList);
	}

	/**
	 * Create the detail band elements
	 * 
	 * @param style
	 *            The style
	 * @return The designband
	 */
	private JRDesignBand createDetailBand(JRDesignStyle style) {
		JRDesignBand detail = new JRDesignBand();
		detail.setSplitType(SplitTypeEnum.STRETCH);
		detail.setHeight(12);

		JRDesignTextField detailTextField1 = createTextField(10, 0, 200, 12, "detailField1", String.class.getName(), "$F{carModel}", HorizontalAlignEnum.LEFT, style);

		JRDesignTextField detailTextField2 = createTextField(210, 0, 280, 12, "detailField2", String.class.getName(), "$F{carColor}", HorizontalAlignEnum.LEFT, style);

		detail.addElement(detailTextField1);
		detail.addElement(detailTextField2);
		return (detail);
	}

	/**
	 * Creates the owner group
	 * 
	 * @param titleStyle
	 *            The group title style
	 * @param headerStyle
	 *            The column header style
	 * @param footerStyle
	 *            The column footer style
	 * @return The owner Group
	 */
	private JRDesignGroup createGroup(JRDesignStyle titleStyle, JRDesignStyle headerStyle, JRDesignStyle footerStyle) {
		JRDesignGroup group = new JRDesignGroup();
		// The group name
		group.setName("CarOwnerGroup");
		// The group expression
		JRDesignExpression groupExpression = new JRDesignExpression();
		groupExpression.setText("$F{personId}");
		groupExpression.setValueClass(Integer.class);
		group.setExpression(groupExpression);

		// Group header
		JRDesignBand groupHeader = createGroupHeaderBand(titleStyle, headerStyle);
		((JRDesignSection) group.getGroupHeaderSection()).addBand(groupHeader);
		JRDesignBand groupFooter = createGroupFooterBand(footerStyle);
		((JRDesignSection) group.getGroupFooterSection()).addBand(groupFooter);

		return group;
	}

	/**
	 * Create the group header band elements
	 * 
	 * @param titleStyle
	 *            The group title style
	 * @param headerStyle
	 *            The column header style
	 * @return The designband
	 */
	private JRDesignBand createGroupHeaderBand(JRDesignStyle titleStyle, JRDesignStyle headerStyle) {

		JRDesignBand header = new JRDesignBand();
		header.setSplitType(SplitTypeEnum.STRETCH);
		header.setHeight(26);

		// group title
		JRDesignFrame titleFrame = createFrame(0, 0, 500, 12, "titleFrame");

		JRDesignTextField groupTitleTextField = createTextField(0, 0, 500, 12, "groupTitleField", String.class.getName(), "\"Owner name: \"+$F{personName}", HorizontalAlignEnum.LEFT, titleStyle,
				false);
		titleFrame.addElement(groupTitleTextField);

		// report columns
		JRDesignFrame headerFrame = createFrame(0, 14, 500, 12, "headerFrame");

		JRDesignTextField headerTextField1 = createTextField(10, 0, 200, 12, "headerField1", String.class.getName(), "\"Car model\"", HorizontalAlignEnum.CENTER, headerStyle);

		JRDesignTextField headerTextField2 = createTextField(210, 0, 280, 12, "headerField2", String.class.getName(), "\"Car color\"", HorizontalAlignEnum.CENTER, headerStyle);

		headerFrame.addElement(headerTextField1);
		headerFrame.addElement(headerTextField2);

		header.addElement(titleFrame);
		header.addElement(headerFrame);
		return (header);
	}

	/**
	 * Create the group footer band elements
	 * 
	 * @param style
	 *            The content style
	 * @return The designband
	 */
	private JRDesignBand createGroupFooterBand(JRDesignStyle style) {

		JRDesignBand footer = new JRDesignBand();
		footer.setSplitType(SplitTypeEnum.STRETCH);
		footer.setHeight(24);

		// group title

		JRDesignTextField groupFooterLabel = createTextField(350, 6, 100, 12, "groupFooterLabel", String.class.getName(), "\"Total cars: \"", HorizontalAlignEnum.LEFT, style);

		JRDesignTextField groupFooterCounter = createTextField(450, 6, 40, 12, "groupFooterCounter", Integer.class.getName(), "$V{CarOwnerGroup_COUNT}", HorizontalAlignEnum.RIGHT, style);

		footer.addElement(groupFooterLabel);
		footer.addElement(groupFooterCounter);
		return (footer);
	}

}